/* Copyright  (C)  2007  Ruben Smits <ruben dot smits at mech dot kuleuven dot be> */

/* Version: 1.0 */
/* Author: Ruben Smits <ruben dot smits at mech dot kuleuven dot be> */
/* Maintainer: Ruben Smits <ruben dot smits at mech dot kuleuven dot be> */
/* URL: http://www.orocos.org/kdl */

/* This library is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU Lesser General Public */
/* License as published by the Free Software Foundation; either */
/* version 2.1 of the License, or (at your option) any later version. */

/* This library is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU */
/* Lesser General Public License for more details. */

/* You should have received a copy of the GNU Lesser General Public */
/* License along with this library; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */


/* Methods are inlined */
#define KDL_INLINE 1

/* Column width that is used form printing frames */
#define KDL_FRAME_WIDTH 12

/* Indices are checked when accessing members of the objects */
#define KDL_INDEX_CHECK 1

/* use KDL implementation for == operator */
#define KDL_USE_EQUAL 1
